@@ -752,7 +752,7 @@ class TileTopology(object):
         if left is not None and self._tile_coord(left)[0] != tile_row:
             left = None
         right = identifier + 1 if identifier < tile_count else None
-        # check whether the tile is on an edge. In this case no left tile.
+        # check whether the tile is on an edge. In this case no right tile.
         if right is not None and self._tile_coord(right)[0] != tile_row:
             right = None
         return top, bottom, left, right
@@ -899,3 +899,111 @@ class TileTopology(object):
     def __len__(self):
         """Number of tiles"""
         return self.tile_count
+
+
+def _borders_have_expected_size(topology, max_width, max_height):
+    """Evaluates whether the border tiles should be skipped
+    Parameters
+    ----------
+    topology: TileTopology
+        A regular tile topology
+    max_width: int
+        Max tile width
+    max_height: int
+        Max tile height
+
+    Returns
+    -------
+    right_not_ok: bool
+        True if right border tiles do not have dimensions (max_width, max_height)
+    bottom_not_ok: bool
+        True if bottom border tiles do not have dimensions (max_width, max_height)
+    """
+    n_h = topology.tile_horizontal_count
+    n_v = topology.tile_vertical_count
+    h_tile = topology.tile(n_h)
+    v_tile = topology.tile((n_v - 1) * n_h + 1)
+    return (h_tile.width != max_width or h_tile.height != max_height,
+            v_tile.width != max_width or v_tile.height != max_height)
+
+
+class SkipBordersTileTopology(TileTopology):
+    """A topology that ensures that all tiles have the specified dimensions. This is done by skipping tiles at right
+    and bottom borders."""
+    def __init__(self, topology):
+        """
+        Parameters
+        ----------
+        topology: TileTopology
+            A regular topology
+        """
+        super(SkipBordersTileTopology, self).__init__(
+            image=topology._image,
+            tile_builder=topology._tile_builder,
+            max_width=topology._max_width,
+            max_height=topology._max_height,
+            overlap=topology._overlap
+        )
+        self._parent = topology
+        skip_h, skip_v = _borders_have_expected_size(topology, self._max_width, self._max_height)
+        self._skip_h = skip_h
+        self._skip_v = skip_v
+
+    def _convert_id(self, parent_identifier):
+        """Convert a parent tile identifier to the corresponding identifier for this topology.
+        Returns
+        -------
+        identifier: int|None
+            A valid identifier for this topology, None if the parent identifier corresponds to an invalid tile
+        """
+        if parent_identifier is None:
+            return None
+        row, col = self._parent._tile_coord(parent_identifier)
+        # check that tile is actually valid
+        if self._skip_h and col == self._parent.tile_horizontal_count - 1:
+            return None
+        if self._skip_v and row == row == self._parent.tile_vertical_count - 1:
+            return None
+        return parent_identifier - (row if self._skip_h else 0)
+
+    @property
+    def tile_horizontal_count(self):
+        return super().tile_horizontal_count - (1 if self._skip_h else 0)
+
+    @property
+    def tile_vertical_count(self):
+        return super().tile_vertical_count - (1 if self._skip_v else 0)
+
+    def tile_neighbours(self, identifier):
+        row, col = self._tile_coord(identifier)
+        parent_identifier = identifier + (row if self._skip_h else 0)
+        return tuple([self._convert_id(_id) for _id in self._parent.tile_neighbours(parent_identifier)])
+
+
+class FixedSizeTileTopology(TileTopology):
+    """A topology that ensures that all tiles have the specified dimensions. This is done by extending tiles at left
+    and bottom borders to overlap their neighbouring by more than the specified overlap."""
+    def __init__(self, topology):
+        """
+        Parameters
+        ----------
+        topology: TileTopology
+            A regular topology
+        """
+        super(FixedSizeTileTopology, self).__init__(
+            image=topology._image,
+            tile_builder=topology._tile_builder,
+            max_width=topology._max_width,
+            max_height=topology._max_height,
+            overlap=topology._overlap
+        )
+
+    def tile_offset(self, identifier):
+        row, col = self._tile_coord(identifier)
+        off_x, off_y = super().tile_offset(identifier)
+        if row == self.tile_vertical_count - 1:
+            off_y -= self._max_height - (self._image.height - off_y)
+        if col == self.tile_horizontal_count - 1:
+            off_x -= self._max_width - (self._image.width - off_x)
+        # take max for when image is too small
+        return max(off_x, 0), max(off_y, 0)
